#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Gestion des utilisateurs
###########################################################################
"""
    Création/modification/suppression des users Posh
    Inscription dans les groupes admin
    from user_api import PoshUserApi
    us = PoshUserApi()
    us.add_user('testuser', 'utilisateur de test')
    us.del_user('testuser')
    us.subscribe_profilgroups('testuser', ['testgroupyetcreated'])
"""
# import système
from os.path import join

# import locaux
from posh.logger.logger import getLogger
from posh.config.api import POSH_API_URL, ACTION_DICT
from posh.error import MissingUserId, MissingGroupId

from request import PoshRequest
from base_api import PoshBaseApi
from group_api import PoshGroupApi
from xml_parse import XmlParse

log = getLogger("api.user_api")

# import Eole
try:
    from creole.eosfunc import gen_random
except ImportError:
    from time import time
    from random import randrange
    from sha import sha
    def gen_random(length=None):
        """
            length: longueur de la chaine aléatoire attendu
        """
        random_id = str(time()).split('.')[0]
        random_str = sha('%s/%s' % (random_id, randrange(2**100))).hexdigest()
        return random_str[:length]

class PoshUserApi(PoshBaseApi):
    """
        Gestionnaire d'utilisateurs posh
    """
    def add_user(self, username, longname, usertype='I', lang='fr'):
        """
            username:nom d'utilisateur
            longname:nom long pour l'utilisateur
            usertype:type d'utilisateur I/A pour internaute/administrateur posh
            lang:langue française (profil par défaut de l'utilisateur)
            1 - Ajout de l'utilisateur
            2 - Ajoute les champs spécifiques
            3I - Ajoute les pages communes à tous
            4I - Ajoute les pages du groupe
            3A - Ajoute les pages admin auxquelles il a droit
            5 - Ajoute l'appartenance aux groupes
        """
        log.debug("# Ajout de l'utilisateur : %s #" % (username,))
        try:
            userid = self.get_userid(username)
            assert userid != None
            log.debug(" + L'utilisateur %s est déjà dans \
la base, son uid : %s" % (username, userid))
        except AssertionError:
            script_url = join(POSH_API_URL, ACTION_DICT['add_user'])
            post_args = dict(username=username,
                             long_name=longname,
                             usertype=usertype,
                             lang=lang,
                             adminkey=self.key)
            password = gen_random(10)
            post_args['pass'] = password
            xml = XmlParse(PoshRequest(script_url, post_args).request())
            userid = xml.userid()
            if xml.success():
                log.debug(" + L'utilisateur %s a bien été ajouté dans \
la base, son uid : %s" % (username, userid))
            else:
                log.error(" - Erreur à l'insertion de l'utilisateur %s dans \
la base" % (username,))
        return userid

    def get_userid(self, username):
        """
            Renvoie l'uid de l'utilisateur,
            username:nom de l'utilisateur
        """
        script_url = join(POSH_API_URL, ACTION_DICT['get_userinfo'])
        post_args = dict(username=username,
                         adminkey=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        try:
            return xml.userid()
        except MissingUserId:
            return None
        return False

    def get_userinfos(self, username):
        """
            Renvoie des infos utilisateurs
            son id,
            son longname,
            sa langue
        """
        script_url = join(POSH_API_URL, ACTION_DICT['get_userinfo'])
        post_args = dict(username=username,
                         adminkey=self.key)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.userinfo()

    def mod_user(self, username, longname, usertype='I', lang='fr'):
        """
            username, userid, long_name, usertype, lang
            attention : Si on met pas un des paramètres, les autres sont remis à 0
            username:Login
            longname:Nom de l'utilisateur
            usertype:A/I admin/internaute
            lang: langue pr défaut de l'utilisateur

            Modifie le nom de l'utilisateur (ou sa langue par défaut) pas son login
        """
        pass

    def subscribe_profilgroups(self, username, groupnames):
        """
            Inscrit un user dans un groupe
            :username: nom de l'utilisateur
            :groupname:nom du groupe
        """
        log.debug("# Inscription du user %s aux groupes %s" % (username,
                                                           str(groupnames)))
        # Recuperation des ids de groupe
        if not hasattr(groupnames, '__iter__'):
            groupnames = [groupnames]
        groupapi = PoshGroupApi()
        try:
            # construction d'un dico {'group0':'id du groupe', 'group1'...}
            groupids = dict([('group%d' % (indice,),
                         groupapi.get_groupid(groupname))
                              for indice, groupname in enumerate(groupnames)])
        except MissingGroupId:
            log.exception(" - Un des groupes parmi %s n'existe pas dans la \
base. Veuillez le créer." % (str(groupnames),))
            groupids = {}
        # recuperation des infos utilisateurs
        # sinon elles sont remises aux valeur par def
        userinfo = self.get_userinfos(username)
        if groupids:
            # inscription aux groupes
            script_url = join(POSH_API_URL, ACTION_DICT['mod_user'])
            post_args = dict(adminkey=self.key, username=username)
            post_args.update(groupids)
            post_args.update(userinfo)
            xml = XmlParse(PoshRequest(script_url, post_args).request())
            if xml.success():
                log.debug(" + L'utilisateur %s a bien été inscrit aux \
groupes %s" % (username, str(groupnames)))
                return True
            else:
                log.error(" - Une erreur est survenue à l'inscription \
de %s dans un des groupes : %s" % (username, str(groupnames)))
                return False
        return False

    def subscribe_profilgroup(self, username, groupid):
        """
            inscrit un utilisateur à un groupe
            :uid: id de l'utilisateur dans la base posh
            :gid: id du groupe dans la base
        """
        log.debug(" + Inscription de l'utilisateur \
%s dans le groupe d'id : %s" % (username, groupid,))
        userinfo = self.get_userinfos(username)
        script_url = join(POSH_API_URL, ACTION_DICT['mod_user'])
        post_args = dict(adminkey=self.key,
                         username=username,
                         group0=groupid)
        post_args.update(userinfo)
        xml = XmlParse(PoshRequest(script_url, post_args).request())
        return xml.success()

    def del_user(self, username):
        """
            Supprime un utilisateur
            username: login de l'utilisateur
        """
        log.debug("# Suppression de l'utilisateur %s de la base Posh \
#" % (username,))
        userid = self.get_userid(username)
        if userid:
            script_url = join(POSH_API_URL, ACTION_DICT['del_user'])
            post_args = dict(action='delete',
                             adminkey=self.key)
            post_args['id'] = userid
            xml = XmlParse(PoshRequest(script_url, post_args).request())
            if xml.success():
                log.debug(" + L'utilisateur %s a bien été supprimé." % (
                                                              username,))
                return True
            else:
                log.error(" - Une erreur est survenue à la suppression : \
%s" % (username,))
                return False
        log.error(" - Cet utilisateur n'existe pas %s." % (username,))
        return False
